/* $Id: PanelMetaCache.java 18181 2010-03-28 14:20:44Z linus $
 *****************************************************************************
 * Copyright (c) 2009 Contributors - see below
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    bobtarling
 *****************************************************************************
 *
 * Some portions of this file was previously release using the BSD License:
 */

// Copyright (c) 2008 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation without fee, and without a written
// agreement is hereby granted, provided that the above copyright notice
// and this paragraph appear in all copies. This software program and
// documentation are copyrighted by The Regents of the University of
// California. The software program and documentation are supplied "AS
// IS", without any accompanying services from The Regents. The Regents
// does not warrant that the operation of the program will be
// uninterrupted or error-free. The end-user understands that the program
// was developed for research purposes and is advised not to rely
// exclusively on the program for any reason. IN NO EVENT SHALL THE
// UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
// SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
// ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
// THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
// PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
// UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

package org.argouml.core.propertypanels.meta;

import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import org.argouml.model.Model;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * The cache of property panel metadata
 *
 * @author Bob Tarling
 */
public class PanelMetaCache {
    
    private final Map<String, PanelMeta> cache =
        new HashMap<String, PanelMeta>();
    

    public PanelMetaCache() throws Exception {
        parseXML();
    }
    
    public PanelMeta get(String name) {
        return cache.get(name);
    }
    
    private void parseXML() throws Exception {
        final String file;        
        if (Model.getFacade().getUmlVersion().charAt(0) == '2') {
            file = "org/argouml/core/propertypanels/meta/panels2.xml";
        } else {
            file = "org/argouml/core/propertypanels/meta/panels.xml";        
        }
        final XMLReader parser = XMLReaderFactory.createXMLReader();
        parser.setContentHandler(new XmlSinglePanelHandler(cache));
        final InputStream stream =
            this.getClass().getClassLoader().getResourceAsStream(file);
        if (stream != null) {
            final InputSource source = new InputSource(stream);
            parser.parse(source);
            stream.close();
        } else {
            throw new IllegalStateException("Failed to find the panel XML");
        }
    }
}
